#!/usr/bin/env python3

# *************************************************************************
#
#  Copyright (c) 2025 - Datatailr Inc.
#  All Rights Reserved.
#
#  This file is part of Datatailr and subject to the terms and conditions
#  defined in 'LICENSE.txt'. Unauthorized copying and/or distribution
#  of this file, in parts or full, via any medium is strictly prohibited.
# *************************************************************************

import os
import sys
import runpy
from importlib.resources import files

from datatailr.logging import DatatailrLogger


logger = DatatailrLogger(os.path.abspath(__file__)).get_logger()


def run():
    logger.info("Starting Datatailr app...")
    entrypoint = os.environ.get("DATATAILR_ENTRYPOINT")
    if entrypoint is None or ":" not in entrypoint:
        raise ValueError(
            "Environment variable 'DATATAILR_ENTRYPOINT' is not in the format 'module_name:file_name'."
        )

    module_name, file_name = entrypoint.split(":")

    script = files(module_name).joinpath(file_name)
    sys.argv = ["streamlit", "run", str(script), *sys.argv[1:]]
    logger.info(f"Running entrypoint: {entrypoint}")
    runpy.run_module("streamlit", run_name="__main__")



try:
    from dt.excel import Addin  # type: ignore
except ImportError:

    class DummyAddin:
        def __init__(self, name: str, description: str) -> None:
            self.name = name
            self.description = description

        def expose(self, **kwargs):
            def decorator(func):
                return func

            return decorator

        def run(self):
            pass

    Addin = DummyAddin



#!/usr/bin/env python3

# *************************************************************************
#
#  Copyright (c) 2025 - Datatailr Inc.
#  All Rights Reserved.
#
#  This file is part of Datatailr and subject to the terms and conditions
#  defined in 'LICENSE.txt'. Unauthorized copying and/or distribution
#  of this file, in parts or full, via any medium is strictly prohibited.
# *************************************************************************


# The purpose of this script is to be the entrypoint for all jobs running on datatailr.
# The main functions of the script are:
#     1. Create a linux user and group for the job.
#     2. Set the environment variables for the job.
#     3. Run the job in a separate process, as the newly created user and pass all relevant environment variables.
# There are muliple environment variables which are required for the job to run.
# Some of them are necessary for the setup stage, which is executed directly in this script as the linux root user.
# Others are passed to the job script, which is executed in a separate process with only the users' privileges and not as a root user.
#
# Setup environment variables:
#     DATATAILR_USER - the user under which the job will run.
#     DATATAILR_GROUP - the group under which the job will run.
#     DATATAILR_UID - the user ID of the user as it is defined in the system.
#     DATATAILR_GID - the group ID of the group as it is defined in the system.
#     DATATAILR_JOB_TYPE - the type of job to run. (batch\service\app\excel\IDE)
# Job environment variables (not all are always relevant, depending on the job type):
#     DATATAILR_BATCH_RUN_ID - the unique identifier for the batch run.
#     DATATAILR_BATCH_ID - the unique identifier for the batch.
#     DATATAILR_JOB_ID - the unique identifier for the job.


import subprocess
import os
import sys
from typing import Tuple
from datatailr.logging import DatatailrLogger
from datatailr.utils import is_dt_installed

logger = DatatailrLogger(os.path.abspath(__file__)).get_logger()

if not is_dt_installed():
    logger.error("Datatailr is not installed.")
    # sys.exit(1) # TODO: Uncomment after testing


def get_env_var(name: str, default: str | None = None) -> str:
    """
    Get an environment variable.
    If the variable is not set, raise an error.
    """
    if name not in os.environ:
        if default is not None:
            return default
        logger.error(f"Environment variable '{name}' is not set.")
        raise ValueError(f"Environment variable '{name}' is not set.")
    return os.environ[name]


def create_user_and_group() -> Tuple[str, str]:
    """
    Create a user and group for the job.
    The user and group names are taken from the environment variables DATATAILR_USER and DATATAILR_GROUP.
    The group and user are created with the same uid and gid as passed in the environment variables DATATAILR_UID and DATATAILR_GID.
    If the user or group already exists, do nothing.
    """
    user = get_env_var("DATATAILR_USER")
    group = get_env_var("DATATAILR_GROUP")
    uid = get_env_var("DATATAILR_UID")
    gid = get_env_var("DATATAILR_GID")

    # Create group if it does not exist
    os.system(f"getent group {group} || groupadd {group} -g {gid} -o")

    # Create user if it does not exist
    os.system(
        f"getent passwd {user} || useradd -g {group} -s /bin/bash -m {user} -u {uid} -o"
    )
    return user, group


def run_command_as_user(command: str | list, user: str, env_vars: dict):
    """
    Run a command as a specific user with the given environment variables.
    """
    if isinstance(command, list):
        command = " ".join(command)
    env_vars = {
        "PATH": get_env_var("PATH", ""),
        "PYTHONPATH": get_env_var("PYTHONPATH", ""),
    } | env_vars

    env_kv = [f"{k}={v}" for k, v in env_vars.items()]
    argv = ["sudo", "-u", user, "env", *env_kv, "bash", "-lc", command]

    try:
        result = subprocess.run(
            argv,
            check=True,  # raises if returncode != 0
            capture_output=True,
            text=True,
        )
        logger.info(f"stdout: {result.stdout}")
        logger.debug(f"stderr: {result.stderr}")
    except subprocess.CalledProcessError as e:
        logger.error(f"Command failed with exit code {e.returncode}")
        logger.error(f"stderr: {e.stderr}")
        sys.exit(1)


def main():
    user, _ = create_user_and_group()
    job_type = get_env_var("DATATAILR_JOB_TYPE")

    env = {
        "DATATAILR_JOB_TYPE": job_type,
        "DATATAILR_JOB_NAME": get_env_var("DATATAILR_JOB_NAME"),
        "DATATAILR_JOB_ID": get_env_var("DATATAILR_JOB_ID"),
    }

    if job_type == "batch":
        run_id = get_env_var("DATATAILR_BATCH_RUN_ID")
        batch_id = get_env_var("DATATAILR_BATCH_ID")
        entrypoint = get_env_var("DATATAILR_BATCH_ENTRYPOINT")
        env = {
            "DATATAILR_BATCH_RUN_ID": run_id,
            "DATATAILR_BATCH_ID": batch_id,
            "DATATAILR_BATCH_ENTRYPOINT": entrypoint,
        } | env
        run_command_as_user("datatailr_run_batch", user, env)
    elif job_type == "service":
        port = get_env_var("DATATAILR_SERVICE_PORT")
        entrypoint = get_env_var("DATATAILR_ENTRYPOINT")
        env = {
            "DATATAILR_ENTRYPOINT": entrypoint,
            "DATATAILR_SERVICE_PORT": port,
        } | env
        run_command_as_user("datatailr_run_service", user, env)
    elif job_type == "app":
        entrypoint = get_env_var("DATATAILR_ENTRYPOINT")
        env = {
            "DATATAILR_ENTRYPOINT": entrypoint,
        } | env
        run_command_as_user("datatailr_run_app", user, env)
    elif job_type == "excel":
        host = get_env_var("DATATAILR_HOST", "")
        entrypoint = get_env_var("DATATAILR_ENTRYPOINT")
        local = get_env_var("DATATAILR_LOCAL", False)
        env = {
            "DATATAILR_ENTRYPOINT": entrypoint,
            "DATATAILR_HOST": host,
            "DATATAILR_LOCAL": local,
        } | env
        run_command_as_user("datatailr_run_excel", user, env)
    elif job_type == "ide":
        command = [
            "code-server",
            "--auth=none",
            "--bind-addr=0.0.0.0:8080",
            f'--app-name="Datatailr IDE {get_env_var("DATATAILR_USER")}"',
        ]
        run_command_as_user(command, user, env)
    elif job_type == "jupyter":
        command = [
            "jupyter-lab",
            "--ip='*'",
            "--port=8080",
            "--no-browser",
            "--NotebookApp.token=''",
            "--NotebookApp.password=''",
        ]
        run_command_as_user(command, user, env)
    else:
        raise ValueError(f"Unknown job type: {job_type}")


if __name__ == "__main__":
    try:
        logger.debug("Starting job execution...")
        main()
        logger.debug("Job executed successfully.")
    except Exception as e:
        logger.error(f"Error during job execution: {e}")
        raise



##########################################################################
#
#  Copyright (c) 2025 - Datatailr Inc.
#  All Rights Reserved.
#
#  This file is part of Datatailr and subject to the terms and conditions
#  defined in 'LICENSE.txt'. Unauthorized copying and/or distribution
#  of this file, in parts or full, via any medium is strictly prohibited.
##########################################################################

from typing import Any
from datatailr.wrapper import dt__Job
import re


__CLIENT__ = dt__Job()


class Schedule:
    """
    Represents a schedule for batch jobs.
    """

    def __init__(
        self,
        cron_expression: str = "",
        at_minutes: list[int] | None = None,
        every_minute: int | None = None,
        at_hours: list[int] | None = None,
        every_hour: int | None = None,
        weekdays: list[str] | None = None,
        day_of_month: int | None = None,
        in_month: list[str] | None = None,
        every_month: int | None = None,
        timezone: str | None = None,
        run_after_job_uuid: str | None = None,
        run_after_job_name: str | None = None,
        run_after_job_condition: str | None = None,
    ):
        self.at_minutes = at_minutes
        self.every_minute = every_minute
        self.at_hours = at_hours
        self.every_hour = every_hour
        self.weekdays = weekdays
        self.day_of_month = day_of_month
        self.in_month = in_month
        self.every_month = every_month
        self.timezone = timezone
        self.run_after_job_uuid = run_after_job_uuid
        self.run_after_job_name = run_after_job_name
        self.run_after_job_condition = run_after_job_condition
        self.schedule_expression = None
        self.cron_expression = cron_expression

        self.__is_set__ = False

    def __str__(self) -> str:
        self.__compile__()
        return self.cron_expression

    def __repr__(self) -> str:
        self.__compile__()
        return f"Schedule(cron_expression={self.cron_expression}, timezone={self.timezone}) - {self.schedule_expression}"

    def __setattr__(self, name: str, value: Any) -> None:
        super().__setattr__(name, value)
        if name in [
            "at_minutes",
            "at_hours",
            "every_minute",
            "every_hour",
            "weekdays",
            "day_of_month",
            "in_month",
            "every_month",
        ]:
            self.__is_set__ = False

    def __compile__(self):
        if self.__is_set__:
            return
        argument_name = [
            "at_minutes",
            "at_hours",
            "every_minute",
            "every_hour",
            "weekdays",
            "day_of_month",
            "in_month",
            "every_month",
            "run_after_job_uuid",
            "run_after_job_name",
            "run_after_job_condition",
        ]
        arguments = {}

        for key in argument_name:
            if hasattr(self, key) and getattr(self, key) is not None:
                value = getattr(self, key)
                if isinstance(value, list):
                    value = ",".join(map(str, value))
                arguments[key] = value

        result = __CLIENT__.run("", cron_string=True, **arguments)
        match = re.match(r"^(.*?)\s*\((.*?)\)$", result)
        if match:
            cron_expression, schedule_expression = match.groups()
            self.cron_expression = "0 " + cron_expression.strip()
            self.schedule_expression = schedule_expression.strip()
        self.__is_set__ = True

    def get_cron_string(self) -> str:
        """
        Returns the compiled cron string.
        """
        self.__compile__()
        return self.cron_expression
